(in-package #:org-syntax-language)

;; We do not ensure uniqueness of instances
;; as a Lisp language may change
;; For example, it makes sense to have several instances of Emacs Lisp because of emacs-version, live Emacs version from official repo, or a fork

;; Still, oftentimes, there is a canonical version, so we implement ensure

;; (deftype lang-designator () (or string symbol standard-class))

(defvar instances nil
  ;; type: (alist lang-designator standard-object)
  )

;; For now, we do not even provide a root class.

(defvar classes nil
  ;; type: (alist string standard-class)
  )

(defun get-lang-class-by-name (string)
  (cdr (assoc string classes :test #'string=)))
(defun register-lang-class (lang-name class)
  (aif (member lang-name classes :key #'car :test #'string=)
       (progn
         (warn "Lang with name ~S already registered~&" lang-name)
         it)
       (push (cons lang-name class) classes)))

(defun ensure (lang-name-or-class &rest default-initargs)
  "Return a lang instance corresponding to lang-name-or-class, if it exists, otherwise return a fresh one.

default-initargs are ignored if instance exists.

#'ensure is featureful rather than safe:
With it, you can get “unique” but different instances corresponding to
- a symbol 'common-lisp
- a string \"common-lisp\", if you register such
- a bunch of different classes named 'common-lisp, if you had redefined the class"
  (flet ((ensure/eq (lang-class)
           (with-memoization (alist-get lang-class instances nil nil #'eq)
             (apply #'make-instance lang-class default-initargs)))
         (ensure/string= (lang-name)
           (with-memoization (alist-get lang-name instances nil nil #'string=)
             (apply #'make-instance (or (get-lang-class-by-name lang-name)
                                        (error "A class corresponding to lang name ~S is not defined"
                                               lang-name))
                    default-initargs))))
    (etypecase lang-name-or-class
      ((or symbol standard-class) (ensure/eq lang-name-or-class))
      (string                     (ensure/string= lang-name-or-class)))))

