;;;; org-syntax.commercial.asd

(eval-when (:compile-toplevel :load-toplevel :execute)
  ;; This eval-when was in literate-lisp, I'm not sure why.
  ;; TODO: Clean up
  (unless (find-package :org-syntax.system)
    (defpackage #:org-syntax.system
      (:use :cl :asdf)
      (:export #:*debug* #:when-debug))))

(in-package #:org-syntax.system)

;; BUG: This symbol is not easily available until the system is compiled or source-loaded
(defvar type-checkers/list-of nil)
(defvar *debug* nil)
(defmacro when-debug (&body body)
  (when *debug*
    `(when *debug* ,@body)))

;; One big problem: swank generally associates readtable with a package
;; It should be associating it with a file

;; Another issue is, SBCL does not simply record the position in file
;; a form was read from; it records something else
;; which makes swank actually read the file
;; and it often won't find the readtable correctly,
;; partially because it associates readtables with packages and not with files.

(asdf:defsystem #:org-syntax.commercial
  :author "Dima Akater <nuclearspace@gmail.com>"
  :licence "Unlicense"
  :version "20250430"
  :serial t
  :description "Read Lisp with Org syntax (a fork of org-syntax for commercial use)"
  :components ((:file "packages")
               (:file "util")
               (:file "languages")
               (:file "syntax")
               (:file "obl.common-lisp")
               (:file "asdf")))

(defmethod asdf::around-compile-hook ((os (eql (find-system "org-syntax.commercial"))))
  (declare (ignore os))
  (if *debug* (lambda (next)
                (proclaim '(optimize
			    (compilation-speed 0)
                            (debug 3)
                            (safety 3)
                            (space 0)
                            (speed 0)))
                (let ((*debug* t))
                  (funcall next)))
    (lambda (next)
      (proclaim '(optimize
		  (compilation-speed 0)
                  (debug 0)
                  (safety 1)
                  (space 0)
                  (speed 3)))
      (funcall next))))

#+akater-types
(defmacro fbind-list-of-checkers ()
  `(progn
     ,@(loop for (name . lambda-expr) in type-checkers/list-of
             do (format t "Preserving ~S for ~S at macroexpansion time~&"
                        name lambda-expr)
             collect `(unless (fboundp ',name)
                        (format t "Compiling ~S~&" ',name)
                        (compile ',name ,lambda-expr)))))

#+akater-types
(defmethod asdf::operate :before ((o asdf::load-op)
                                  (c (eql (find-system "org-syntax.commercial")))
                                  &rest _)
  (declare (ignore _))
  (loop for (name . lambda-expr) in type-checkers/list-of
        do (if (fboundp name) (format t "~S is fbound~&" name)
               (progn
                 (format t "Compiling ~S~&" name)
                 (compile name lambda-expr)))))

#+akater-types
(defmethod asdf::operate :after ((o asdf::compile-op)
                                 (c (eql (find-system "org-syntax.commercial")))
                                 &rest _)
  (declare (ignore _))
  (loop for (name . lambda-expr) in type-checkers/list-of
        do (unless (fboundp name) (format t "Compiling ~S~&" name) (compile name lambda-expr))))
